// Thomas Grenier - Creatis

nb_Xi = 100; // nombre de tirages
res_F = 100; // rsolution des tracs
var_x = linspace( 0, 20, res_F); // interval pour le calcul des densits


// Loi gamma (si scale = 1 => poisson)
shape = 5;
scale = 1;
Xi = [grand( nb_Xi, 1, 'gam', shape, scale)]; 
deff('[z]=poisson(x)','z=(scale^shape)*x^(shape-1)*exp(-scale*x)/gamma(shape)' );
pdf = feval( var_x, poisson);

// Loi exponentielle
//Xi=[grand( nb_Xi, 1, 'exp', 5)];
//deff('[z]=exponentielle(x)','z=exp(-x/scale)/scale' );
//pdf = feval( var_x, exponentielle);
//for i=1:res_F,
//  if( var_x(i) < 0)
//    pdf(i)=0;
//   end;
//end;



// Parzen classique (Gauss)
function [fx]=Parzen_1D(X,Xi,h)
fx=0;
[nb_Xi,dim]=size(Xi);
for i = 1:nb_Xi,
  x_xi = X  - Xi(i);
  dxr = x_xi / h;
  fx = fx + exp(- (dxr*dxr)/2) / sqrt(2*3.14159);
end
fx = fx / (nb_Xi * h );
endfunction

// Parzen Gauss Adaptatif (hi est un vecteur)
function [fx]=ParzenGA_1D( X, Xi, hi)
fx=0;
[nb_Xi,dim]=size(Xi);
for i = 1:nb_Xi,
  x_xi = X  - Xi(i);
  dxr = x_xi / hi(i);
  fx = fx + exp(-dxr * dxr/2) / ( sqrt(2*3.14159)* hi(i) );
end
fx = fx / nb_Xi ;
endfunction

// Parzen Epanech Adaptatif (hi est un vecteur)
function [fx]=ParzenHA_1D( X, Xi, hi)
fx=0;
[nb_Xi,dim]=size(Xi);
for i = 1:nb_Xi,
  x_xi = X  - Xi(i);
  dxr = x_xi / hi(i);
  if(dxr*dxr < 1)
    fx = fx + 3*(1-dxr*dxr)/(4*hi(i));
  end;
end
fx = fx / nb_Xi ;
endfunction



// calcul de h0 pour Gauss
alphaR = (4/(3*nb_Xi))^(1/5);
h0 = alphaR * iqr( Xi,'r')./1.349 // Pour Gauss

// calcul de h0 pour Epanech
//alphaR = (40*gamma(1/2)/nb_Xi)^(1/5);
//h0 = alphaR * iqr(Xi,'r')./1.349;

// calcul de l'estimation pilote f_t 
f_t=ones(nb_Xi,1);
for i=1:nb_Xi,
  f_t(i) = Parzen_1D( Xi(i), Xi, h0);
end;
// calcul de lambda
Lambda = geomean( f_t )
// calcul des paramtres adaptatifs hi
hi = ones(nb_Xi,1);
for i=1:nb_Xi,
  hi(i) = h0*sqrt( Lambda/f_t(i) );
end;

// GRAPH
FA = ones( res_F, 1);
F = ones( res_F, 1);

for i=1:res_F,
  FA(i) = ParzenGA_1D( var_x(i), Xi, hi);
  F(i) = Parzen_1D( var_x(i), Xi, h0);
end;


scf();	
//histplot(30, Xi);//, style=2)

//scf();

//plot(px, feval(px,poisson),'b' )
plot( Xi, 0.02, 'b+')
plot(var_x, F,'g', var_x, FA,'r', var_x, pdf ,'b' );

scf();
plot(Xi, hi, 'x');

