// Thomas Grenier - Creatis

function [ms]=FiltrageMeanShiftEGc( Image, n, h)
// Image : image d'entree
// n : taille de l'echantillon pour l'estimation 
// h : vecteur de parametres d'echelle

// Shoot Xi  qui s'appellent Y ...
[nb_X, nb_Y]= size(Image);
Y=[grand( n , 1 , 'unf', 1, nb_X ), grand( n , 1 , 'unf', 1, nb_Y ), ones(n,1)];
for i=1:n,
  Y(i,3)= Image( Y(i,1), Y(i,2) );
end

ms = Image;

for i=1:nb_X,
  disp(i);
  for j=1:nb_Y,
  a = ProcedureMeanShiftEGc( [i,j, Image(i,j)] , Y, h, 0.1);
  ms(i,j) = a(3);
  end;
end;
endfunction;

function [ms]=FiltrageMeanShiftPruneEGc( Image, n, h, vh)
// Image : image d'entree
// n : taille de l'echantillon pour l'estimation 
// h : vecteur de parametres d'echelle
// vh : Vecteur de secouage 

// Shoot Xi  qui s'appellent Y ...
[nb_X, nb_Y]= size(Image);
Y=[grand( n , 1 , 'unf', 1, nb_X ), grand( n , 1 , 'unf', 1, nb_Y ), ones(n,1)];
for i=1:n,
  Y(i,3)= Image( Y(i,1), Y(i,2) );
end

ms = Image;

for i=1:nb_X,
  disp(i);
  for j=1:nb_Y,
  a = ProcedureMeanShiftEGc( [i,j, Image(i,j)] , Y, h, 0.1);
  c = ProcedureMeanShiftEGc( Prune(a, vh) , Y, h, 0.1);
  k=1;
  while Dist2Mahalanobis( a, c, h) > 0.1
    disp("Prune running "+string(k))
    a = c;
    c = ProcedureMeanShiftEGc( Prune(a, vh) , Y, h, 0.1);
    k = k + 1;
  end 
  ms(i,j) = c(3);
  end;
end;
endfunction;

function d=Dist2Mahalanobis( a, b, h)
  dx = a - b;
  d = dx(1)*dx(1)/(h(1)*h(1)) + dx(2)*dx(2)/(h(2)*h(2)) + dx(3)*dx(3)/(h(3)*h(3));
endfunction;


function [x_n]=ProcedureMeanShiftEGc( x, Xi, h, epsilon)
// x : point  deplacer
// Xi : echantillon pour l'estimation
// h : vecteur d'echelles
// epsilon : distance mini accpetable
// x_n : resultat : x  l'iteration suivante

x_tmp = x;
x_n = DParzen_EGc( x_tmp, Xi, h);
dx = x_tmp - x_n;
d = dx(1)*dx(1)/(h(1)*h(1)) + dx(2)*dx(2)/(h(2)*h(2)) + dx(3)*dx(3)/(h(3)*h(3));
// calcul deplacement 
while  d > epsilon,
  x_n = DParzen_EGc( x_tmp, Xi, h);
  dx = x_tmp - x_n;
  d = dx(1)*dx(1)/(h(1)*h(1)) + dx(2)*dx(2)/(h(2)*h(2)) + dx(3)*dx(3)/(h(3)*h(3));
  x_tmp= x_n;
end;  

endfunction;

function [xp]=Prune( x, vh)
// Secouage de x par un vecteur alatoire inclu dans l'elipse vh
// on tire 3 nombres entiers entre 100(x+-vh)
[dim_x, dim_y]=size(x);
xp = x;
for i=1:dim_y,
  xp(i) = grand(1, 1,'unf',100*(x(i)-vh(i)), 100*(x(i)+vh(i)) ) / 100;
end

endfunction

