// Thomas Grenier - Creatis
// Gaussian NON blurring Mean Shift
// with truncated gaussian kernel  (k(x)=0 if x>3.sigma (3.h^2) )
// the next ones are the ranges features
// check the DParzen_GG to see the feature space vector.
// Example of use : (Ex_AdvGBMS)
//      nb_Xi = 1000;
//      Image = ReadImage('C:\Users\grenier\Documents\Grenier\Scilab\Data\R-G-B64_ColorImage_blurred15_3_sd30_norm.png');
//      dImage = double(Image);
//     [nb_X, nb_Y,nb_Dim] = size(dImage);
//      o_Xi=[grand( nb_Xi , 1 , 'unf', 1, nb_X ), grand( nb_Xi , 1 , 'unf', 1, nb_Y )];
//      good_Xi = zeros( nb_Xi, nb_Dim+2);
//      for i=1:nb_Xi,
//          good_Xi(i,1) = o_Xi(i,1); // spatial information
//          good_Xi(i,2) = o_Xi(i,2);
//          for j=1:nb_Dim
//              good_Xi(i,j+2) = dImage( o_Xi(i,1), o_Xi(i,2), j);
//          end
//      end
//          
//    [As, Ar] =GMS(good_Xi(:,1:2), good_Xi(:,3:5), [3 3], [20 20 20], 0.001, 30);
//
//    figure;
//    ShowColorImage(Image,'Image') ;
//    // WARNING : ShowImage permutes x and y 
//    plot( good_Xi(:,2) , nb_X-good_Xi(:,1)+1, 'b+');
//    for j=1:nb_Xi,
//       plot([As(j,2) good_Xi(j,2)], [nb_X-As(j,1)+1 nb_X-good_Xi(j,1)+1], 'k');    
//    end
//    plot( As(:,2) , nb_X-As(:,1)+1, 'w*');


function [Cis, Cir]=GMS(Xis, Xir, hs, hr, eps_ms, nb_max_it)//, activate_plot)
// Xis : Spatial feature of samples 
// Xir : Range features of samples 
// hs : vector of scale parameter for spatial components (not squared)
// hr : vector of scale parameter for range components (not squared)
// trunc : if <> 0 trunc Gaussian to 3.sigma (3.h)
// activate_plot : to drawn mean shift vectors (huge)
// Cis : spatial components of clusterised point of [Xis Xir]
// Cis : range components of clusterised point of [Xis Xir]
// nb_max_it : maximum number of iterations
// eps_ms : minimum value of average shifts that should be considered as 'significative' change

    Ys  = Xis;
    Yr  = Xir;
    
    [nb_Yis nb_DimS] = size(Ys);
    [nb_Yir nb_DimR] = size(Yr);
    
    Ci = 0
    if( nb_Yis ~= nb_Yir) disp('the sample sets Xis and Xir should have the same number of entries');
        return(Ci);
    end
    
     if( size(hs) ~= nb_DimS) disp('the dimension of hs and Xis should be the same');
        return(Ci);
     end
   
     if( size(hr) ~= nb_DimR) disp('the dimension of hs and Xis should be the same');
        return(Ci);
     end
    
    nb_Yi = nb_Yis;
       
    // new samples
    Yns = Ys;
    Ynr = Yr;
    // original samples
    Yos = Ys;
    Yor = Yr;
    
    iHs = inv(diag(hs.^2));
    iHr = inv(diag(hr.^2));
    
    SumOfMS = 1;
    i=0;
    while ((i<nb_max_it) & (SumOfMS > eps_ms) )  
        i = i + 1;
        SumOfMS = 0;
        for j=1:nb_Yi,
            [Yns(j,:) Ynr(j,:)] = DParzen_GGnD(Ys(j,:), Yr(j,:), Yos, Yor, iHs, iHr, 3);
            MS = [Yns(j,:) Ynr(j,:)] - [Ys(j,:) Yr(j,:)];

            SumOfMS = SumOfMS + MS*MS';
        end;
      
      Ys = Yns;
      Yr = Ynr;
      SumOfMS = (SumOfMS) / nb_Yi;
      disp('GMS It :'+ string(i) + ' for ' + string(SumOfMS));
    end;  
    Cis = Ys;
    Cir = Yr;
endfunction
    